@@ -1,4 +1,5 @@
 test_that("runRSSS", {
+
   f1 <- tempfile()
   f2 <- tempfile()
   f3 <- tempfile()
@@ -8,16 +9,19 @@ test_that("runRSSS", {
 
   d <- loadData(response = f1, itemmap = f2, anchor = f3)
 
-  out_cfa     <- suppressWarnings(runCFA(d))
+  out_cfa      <- suppressWarnings(runCFA(d))
 
-  solution    <- runLinking(d, method = "SL", technical = list(NCYCLES = 1000))
-  score_table <- runRSSS(d, solution)
-  solution    <- runLinking(d, method = "FIXEDPAR")
-  score_table <- runRSSS(d, solution)
+  solution <- runLinking(d, method = "SL", technical = list(NCYCLES = 1000))
+  table_sl <- runRSSS(d, solution)
 
-  solution    <- runEquateObserved(d)
+  solution <- runLinking(d, method = "FIXEDPAR")
+  table_fp <- runRSSS(d, solution)
+
+  table_eq <- runEquateObserved(d)
+  table_eq$concordance
 
   file.remove(f1)
   file.remove(f2)
   file.remove(f3)
+
 })
